// Ansteuerung eines HD44780 kompatiblen LCD im 4-Bit-Interfacemodus
// http://www.mikrocontroller.net/articles/AVR-GCC-Tutorial/LCD-Ansteuerung
//

/*
Abgenderte Version zur Ansteuerung eines LCDs ber den I2C-Porterweiterungsbaustein PCF8574.

Belegung des Verbindungskabels:

LCD-Sockel 14 plg	|	Wannenbuchse 10plg PCF8574

rot (markierte Ader)|	GND
+					|	+
D4					|	P0
LED					|	P7
D5					|	P1
RS					|	P6
D6					|	P2
E					|	P5
D7					|	P3
RW					|	P4

*/
 
#ifndef LCD_ROUTINES_H
#define LCD_ROUTINES_H
 
////////////////////////////////////////////////////////////////////////////////
// Hier die verwendete Taktfrequenz in Hz eintragen, wichtig!
 
#ifndef F_CPU
#define F_CPU 4000000	//im Mega48 ist 8 MHz und Teiler durch 2 eingestellt
#endif
 
////////////////////////////////////////////////////////////////////////////////
// Pinbelegung fr das LCD, an verwendete Pins anpassen
// Alle LCD Pins mssen an einem Port angeschlossen sein und die 4
// Datenleitungen mssen auf aufeinanderfolgenden Pins liegen
 
//  LCD DB4-DB7 <-->  PORTC Bit PC0-PC3
//#define LCD_PORT      PORTC			//LCD_PORT sei ab sofort eine Variable!!! wird in der .c Datei definiert.
//#define LCD_DDR       DDRC			//fllt bei I2C ganz weg
#define LCD_DB        PC0				//so lassen. PC0 hat den Wert 0. D4,D5,D6,D7 werden an P0,P1,P2,P3 angeschlossen.
 
//  LCD RS      <-->  PORTC Bit PC6     (RS: 1=Data, 0=Command)
#define LCD_RS        PC6
 
//  LCD EN      <-->  PORTC Bit PC5     (EN: 1-Impuls fr Daten)
#define LCD_EN        PC5

#define LCD_RW        PC4				//soll vorerst immer low sein, da normalerweise nicht vom LCD gelesen wird.
 
////////////////////////////////////////////////////////////////////////////////
// LCD Ausfhrungszeiten (MS=Millisekunden, US=Mikrosekunden)
 
#define LCD_BOOTUP_MS           15
//#define LCD_BOOTUP_MS           250
#define LCD_ENABLE_US           20	//kann fr das lange Kabel zu kurz sein!! Nein, es hat bei "enable" ein I2C-Befehl gefehlt.
//#define LCD_ENABLE_US           250
#define LCD_WRITEDATA_US        46
//#define LCD_WRITEDATA_US        246
#define LCD_COMMAND_US          42
//#define LCD_COMMAND_US          542

 
#define LCD_SOFT_RESET_MS1      5
#define LCD_SOFT_RESET_MS2      1
#define LCD_SOFT_RESET_MS3      1
#define LCD_SET_4BITMODE_MS     5
 
#define LCD_CLEAR_DISPLAY_MS    2
#define LCD_CURSOR_HOME_MS      2
 
////////////////////////////////////////////////////////////////////////////////
// Zeilendefinitionen des verwendeten LCD
// Die Eintrge hier sollten fr ein LCD mit einer Zeilenlnge von 16 Zeichen passen
// Bei anderen Zeilenlngen mssen diese Eintrge angepasst werden
 
#define LCD_DDADR_LINE1         0x00
#define LCD_DDADR_LINE2         0x40
#define LCD_DDADR_LINE3         0x10
#define LCD_DDADR_LINE4         0x50
 
////////////////////////////////////////////////////////////////////////////////
// Initialisierung: muss ganz am Anfang des Programms aufgerufen werden.
void I2Clcd_init( void );
 
////////////////////////////////////////////////////////////////////////////////
// LCD lschen
void I2Clcd_clear( void );
 
////////////////////////////////////////////////////////////////////////////////
// Cursor in die 1. Zeile, 0-te Spalte
void I2Clcd_home( void );
 
////////////////////////////////////////////////////////////////////////////////
// Cursor an eine beliebige Position 
void I2Clcd_setcursor( uint8_t spalte, uint8_t zeile );
 
////////////////////////////////////////////////////////////////////////////////
// Ausgabe eines einzelnen Zeichens an der aktuellen Cursorposition 
void I2Clcd_data( uint8_t data );
 
////////////////////////////////////////////////////////////////////////////////
// Ausgabe eines Strings an der aktuellen Cursorposition 
void I2Clcd_string( const char *data );
 
////////////////////////////////////////////////////////////////////////////////
// Definition eines benutzerdefinierten Sonderzeichens.
// data muss auf ein Array[5] mit den Spaltencodes des zu definierenden Zeichens
// zeigen
void I2Clcd_generatechar( uint8_t code, const uint8_t *data );
 
////////////////////////////////////////////////////////////////////////////////
// Ausgabe eines Kommandos an das LCD.
void I2Clcd_command( uint8_t data );
 
 
////////////////////////////////////////////////////////////////////////////////
// bergabe der Variablen an den I2C-Bus und Addressierung an den PCF8574
void pcf8574_set_outputs (unsigned char address, unsigned char byte);

////////////////////////////////////////////////////////////////////////////////


// LCD Befehle und Argumente.
// Zur Verwendung in lcd_command
 
// Clear Display -------------- 0b00000001
#define LCD_CLEAR_DISPLAY       0x01
 
// Cursor Home ---------------- 0b0000001x
#define LCD_CURSOR_HOME         0x02
 
// Set Entry Mode ------------- 0b000001xx
#define LCD_SET_ENTRY           0x04
 
#define LCD_ENTRY_DECREASE      0x00
#define LCD_ENTRY_INCREASE      0x02
#define LCD_ENTRY_NOSHIFT       0x00
#define LCD_ENTRY_SHIFT         0x01
 
// Set Display ---------------- 0b00001xxx
#define LCD_SET_DISPLAY         0x08
 
#define LCD_DISPLAY_OFF         0x00
#define LCD_DISPLAY_ON          0x04
#define LCD_CURSOR_OFF          0x00
#define LCD_CURSOR_ON           0x02
#define LCD_BLINKING_OFF        0x00
#define LCD_BLINKING_ON         0x01
 
// Set Shift ------------------ 0b0001xxxx
#define LCD_SET_SHIFT           0x10
 
#define LCD_CURSOR_MOVE         0x00
#define LCD_DISPLAY_SHIFT       0x08
#define LCD_SHIFT_LEFT          0x00
#define LCD_SHIFT_RIGHT         0x04
 
// Set Function --------------- 0b001xxxxx
#define LCD_SET_FUNCTION        0x20
 
#define LCD_FUNCTION_4BIT       0x00
#define LCD_FUNCTION_8BIT       0x10
#define LCD_FUNCTION_1LINE      0x00
#define LCD_FUNCTION_2LINE      0x08
#define LCD_FUNCTION_5X7        0x00
#define LCD_FUNCTION_5X10       0x04
 
#define LCD_SOFT_RESET          0x30
 
// Set CG RAM Address --------- 0b01xxxxxx  (Character Generator RAM)
#define LCD_SET_CGADR           0x40
 
#define LCD_GC_CHAR0            0
#define LCD_GC_CHAR1            1
#define LCD_GC_CHAR2            2
#define LCD_GC_CHAR3            3
#define LCD_GC_CHAR4            4
#define LCD_GC_CHAR5            5
#define LCD_GC_CHAR6            6
#define LCD_GC_CHAR7            7
 
// Set DD RAM Address --------- 0b1xxxxxxx  (Display Data RAM)
#define LCD_SET_DDADR           0x80
 
#endif 
