// Ansteuerung eines HD44780 kompatiblen LCD im 4-Bit-Interfacemodus
// http://www.mikrocontroller.net/articles/HD44780
// http://www.mikrocontroller.net/articles/AVR-GCC-Tutorial/LCD-Ansteuerung
//
// Die Pinbelegung ist ber defines in lcd-routines.h einstellbar
// mit ATmega48 ist die Einstellung der clock/8-fuse relevant. Das Programm luft zu schnell.

/*
Abgenderte Version zur Ansteuerung eines LCDs ber den I2C-Porterweiterungsbaustein PCF8574.


- LCD_PORT abndern in eine Variable.

- LCD_EN
- LCD_RS
- LCD_DB	PC0		hier noch etwas erfinden -->> Wert ist immer 0. Am besten so lassen.

verwendet "pcf8574_set_outputs (0x20, LCD_PORT);" um die Variable ber den I2C-Bus zu schicken.

*/


#include <avr/io.h>
#include "LCD-RoutinesI2C.h"	//Fr einen anderen Typ mu hier die umbenannte Datei eingetragen sein
#include <util/delay.h>

uint8_t LCD_PORT = 0;			//neue Variable, die dann ber I2C an den Porterweiterungsbaustein bergeben wird.
 
////////////////////////////////////////////////////////////////////////////////
// Erzeugt einen Enable-Puls
//static void I2Clcd_enable( void )		//kann man mit "static" nicht von auerhalb aufrufen
void I2Clcd_enable( void )
{
    LCD_PORT |= (1<<LCD_EN);     // Enable auf 1 setzen
	pcf8574_set_outputs (0x20, LCD_PORT);
	
    _delay_us( LCD_ENABLE_US );  // kurze Pause	kann vermutlich entfallen, da der I2C-Buszugriff mindestens genauso lange dauert
    LCD_PORT &= ~(1<<LCD_EN);    // Enable auf 0 setzen
	pcf8574_set_outputs (0x20, LCD_PORT);
}
 
////////////////////////////////////////////////////////////////////////////////
// Sendet eine 4-bit Ausgabeoperation an das LCD
static void I2Clcd_out( uint8_t data )
{
    data &= 0xF0;                       // obere 4 Bit maskieren
 
    LCD_PORT &= ~(0xF0>>(4-LCD_DB));    // Maske lschen
	pcf8574_set_outputs (0x20, LCD_PORT);
	
    LCD_PORT |= (data>>(4-LCD_DB));     // Bits setzen
	pcf8574_set_outputs (0x20, LCD_PORT);
    I2Clcd_enable();
}
 
////////////////////////////////////////////////////////////////////////////////
// Initialisierung: muss ganz am Anfang des Programms aufgerufen werden.
void I2Clcd_init( void )
{
    // verwendete Pins auf Ausgang schalten
    uint8_t pins = (0x0F << LCD_DB) |          // 4 Datenleitungen
                   (1<<LCD_RS) |               // R/S Leitung
                   (1<<LCD_EN) |				// Enable Leitung
				   (1<<LCD_RW);                // RW-Leitung soll immer low sein, so lange man nicht lesen mchte
//    LCD_DDR |= pins;							//fllt bei I2C ganz weg
 
    // initial alle Ausgnge auf Null
    LCD_PORT &= ~pins;
	pcf8574_set_outputs (0x20, LCD_PORT);
 
    // warten auf die Bereitschaft des LCD
    _delay_ms( LCD_BOOTUP_MS );
    
    // Soft-Reset muss 3mal hintereinander gesendet werden zur Initialisierung
    I2Clcd_out( LCD_SOFT_RESET );
    _delay_ms( LCD_SOFT_RESET_MS1 );
 
    I2Clcd_enable();
    _delay_ms( LCD_SOFT_RESET_MS2 );
 
    I2Clcd_enable();
    _delay_ms( LCD_SOFT_RESET_MS3 );
 
    // 4-bit Modus aktivieren 
    I2Clcd_out( LCD_SET_FUNCTION |
             LCD_FUNCTION_4BIT );
    _delay_ms( LCD_SET_4BITMODE_MS );
 
    // 4-bit Modus / 2 Zeilen / 5x7
    I2Clcd_command( LCD_SET_FUNCTION |
                 LCD_FUNCTION_4BIT |
                 LCD_FUNCTION_2LINE |
                 LCD_FUNCTION_5X7 );
 
    // Display ein / Cursor aus / Blinken aus
    I2Clcd_command( LCD_SET_DISPLAY |
                 LCD_DISPLAY_ON |
                 LCD_CURSOR_OFF |
                 LCD_BLINKING_OFF); 
 
    // Cursor inkrement / kein Scrollen
    I2Clcd_command( LCD_SET_ENTRY |
                 LCD_ENTRY_INCREASE |
                 LCD_ENTRY_NOSHIFT );
 
    I2Clcd_clear();
}
  
////////////////////////////////////////////////////////////////////////////////
// Sendet ein Datenbyte an das LCD
void I2Clcd_data( uint8_t data )
{
    LCD_PORT |= (1<<LCD_RS);    // RS auf 1 setzen
	pcf8574_set_outputs (0x20, LCD_PORT);
 
    I2Clcd_out( data );            // zuerst die oberen, 
    I2Clcd_out( data<<4 );         // dann die unteren 4 Bit senden
 
    _delay_us( LCD_WRITEDATA_US );
}
 
////////////////////////////////////////////////////////////////////////////////
// Sendet einen Befehl an das LCD
void I2Clcd_command( uint8_t data )
{
    LCD_PORT &= ~(1<<LCD_RS);    // RS auf 0 setzen
	pcf8574_set_outputs (0x20, LCD_PORT);
 
    I2Clcd_out( data );             // zuerst die oberen, 
    I2Clcd_out( data<<4 );           // dann die unteren 4 Bit senden
 
    _delay_us( LCD_COMMAND_US );
}
 
////////////////////////////////////////////////////////////////////////////////
// Sendet den Befehl zur Lschung des Displays
void I2Clcd_clear( void )
{
    I2Clcd_command( LCD_CLEAR_DISPLAY );
    _delay_ms( LCD_CLEAR_DISPLAY_MS );
}
 
////////////////////////////////////////////////////////////////////////////////
// Sendet den Befehl: Cursor Home
void I2Clcd_home( void )
{
    I2Clcd_command( LCD_CURSOR_HOME );
    _delay_ms( LCD_CURSOR_HOME_MS );
}
 
////////////////////////////////////////////////////////////////////////////////
// Setzt den Cursor in Spalte x (0..15) Zeile y (1..4) 
 
void I2Clcd_setcursor( uint8_t x, uint8_t y )
{
    uint8_t data;
 
    switch (y)
    {
        case 1:    // 1. Zeile
            data = LCD_SET_DDADR + LCD_DDADR_LINE1 + x;
            break;
 
        case 2:    // 2. Zeile
            data = LCD_SET_DDADR + LCD_DDADR_LINE2 + x;
            break;
 
        case 3:    // 3. Zeile
            data = LCD_SET_DDADR + LCD_DDADR_LINE3 + x;
            break;
 
        case 4:    // 4. Zeile
            data = LCD_SET_DDADR + LCD_DDADR_LINE4 + x;
            break;
 
        default:
            return;                                   // fr den Fall einer falschen Zeile
    }
 
    I2Clcd_command( data );
}
 
////////////////////////////////////////////////////////////////////////////////
// Schreibt einen String auf das LCD
 
void I2Clcd_string( const char *data )
{
    while( *data != '\0' )
        I2Clcd_data( *data++ );
}
 
////////////////////////////////////////////////////////////////////////////////
// Schreibt ein Zeichen in den Character Generator RAM
 
void I2Clcd_generatechar( uint8_t code, const uint8_t *data )
{
    // Startposition des Zeichens einstellen
    I2Clcd_command( LCD_SET_CGADR | (code<<3) );
 
    // Bitmuster bertragen
    for ( uint8_t i=0; i<8; i++ )
    {
        I2Clcd_data( data[i] );
    }
}
